﻿//dedicated to maestros

using System;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.Device.Location;
using System.Linq;
using System.Windows;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Controls.Maps;
using Microsoft.Phone.Reactive;
using System.Windows.Controls;
using System.Windows.Media;

namespace googlemaps
{
    public partial class MainPage : PhoneApplicationPage
    {
        //랜덤
        private readonly Random random = new Random();
        //위치 정보 조회용
        private readonly GeoCoordinateWatcher geoCoordinateWatcher = new GeoCoordinateWatcher();
        //핀 추가 레이어
        MapLayer layer = new MapLayer();
        //핀id
        int pinCount;
        //핀 좌표 컬렉션
        private ObservableCollection<GeoCoordinate> geoCollection = new ObservableCollection<GeoCoordinate>();
        //구독취소 테스트용
        IDisposable one;

        /// <summary>
        /// 생성자
        /// </summary>
        public MainPage()
        {
            InitializeComponent();
            street.Visibility = Visibility.Visible;

            pinCount = 0;
            one = null;

            Loaded += OnLoaded;
        }

        // 화면 로드 이벤트
        private void OnLoaded(object sender, RoutedEventArgs e)
        {
            //지도 줌 레벨 설정
            googlemap.ZoomLevel = 13;

            //좌표를 감시하는 변수 - 스테틱
            var observable = CreateObservableGeoPositionWatcher();
            //그 변수의 감시 결과를 구독 - 스테틱
            observable
                .ObserveOn(Scheduler.ThreadPool)
                .ObserveOn(Scheduler.Dispatcher)
                .Subscribe(geo => 
                {
                    Latitude.Text = geo.Latitude.ToString();
                    Longitude.Text = geo.Longitude.ToString();

                    googlemap.Center = geo;
                });

            //핀 좌표 컬렉션 감시 - 스테틱
            Observable
                .FromEvent<NotifyCollectionChangedEventArgs>(geoCollection, "CollectionChanged")
                .ObserveOn(Scheduler.ThreadPool)
                .ObserveOn(Scheduler.Dispatcher)
                .Subscribe(evt => 
                {
                    //핀 좌표 변경 시 처리
                    switch (evt.EventArgs.Action)
                    {
                        case NotifyCollectionChangedAction.Add:
                            pinCount++;
                            var location = evt.EventArgs.NewItems[0] as GeoCoordinate;
                            //핀생성
                            Pushpin pin = new Pushpin 
                            { 
                                Location = location, 
                                Content = pinCount.ToString(),
                                Background = location.Speed == 1 ? new SolidColorBrush(Colors.Blue) : new SolidColorBrush(Colors.Red)
                            };
                            
                            layer.Children.Add(pin);
                            break;
                        case NotifyCollectionChangedAction.Remove:
                            //핀제거
                            GeoCoordinate ungeo = evt.EventArgs.OldItems[0] as GeoCoordinate;
                            Pushpin unpin = layer
                                            .Children.Cast<Pushpin>()
                                            .SingleOrDefault(
                                                p => p.Location.Latitude == ungeo.Latitude
                                                    && p.Location.Longitude == ungeo.Longitude);
                            if (unpin != null)
                            {
                                layer.Children.Remove(unpin);
                            }
                            break;
                        case NotifyCollectionChangedAction.Reset:
                            layer.Children.Clear();
                            break;
                    }
                });
            //핀 레이어 구글 맵에 추가
            googlemap.Children.Add(layer);
            //위치 정보 조회 시작
            geoCoordinateWatcher.Start();
        }

        //위치정보 좌표 반환 변수 생성
        private IObservable<GeoCoordinate> CreateObservableGeoPositionWatcher()
        {
            //geoCoordinateWatcher 녀석의 PositionChanged 를 감시 한다.
            var observable = Observable.FromEvent<GeoPositionChangedEventArgs<GeoCoordinate>>(
                e => geoCoordinateWatcher.PositionChanged += e,
                e => geoCoordinateWatcher.PositionChanged -= e)
                .Select(e => e.EventArgs.Position.Location);

            return observable;
        }

        //줌인
        private void ButtonZoomIn_Click(object sender, System.Windows.RoutedEventArgs e)
        {
        	googlemap.ZoomLevel++;
        }
        //줌아웃
        private void ButtonZoomOut_Click(object sender, System.Windows.RoutedEventArgs e)
        {
        	googlemap.ZoomLevel--;
        }
        //Pin, 1, 2 버튼
        private void Button_Click(object sender, System.Windows.RoutedEventArgs e)
        {
            Button ctn = sender as Button;

            switch (ctn.Content.ToString())
            { 
                case "Pin":
                    //현재 위치 Pin만들기
                    var pin = new GeoCoordinate { Latitude = Convert.ToDouble(Latitude.Text), 
                                                  Longitude = Convert.ToDouble(Longitude.Text) };
                    geoCollection.Add(pin);
                    break;
                case "1":
                    //1초에 한번씩 발생 - 현재 스레드
                    var tik = Observable
                                .Interval(new TimeSpan(0, 0, 1));

                    //구독 증명;;
                    one = tik.Subscribe(time => 
                            {
                                var latitude = ((random.NextDouble() * 0.26114931) - 0.0) + 37.4;
                                var longitude = ((random.NextDouble() * 0.30761719) - 0.0) + 126.8;
                                var pin2 = new GeoCoordinate
                                {
                                    Latitude = Convert.ToDouble(latitude),
                                    Longitude = Convert.ToDouble(longitude),
                                    Speed = 1
                                };
                                geoCollection.Add(pin2);
                            });

                    //3초에 한번씩 발생 - 스레드 독립 - 여긴 평생 구독;;
                    Observable
                        .Interval(new TimeSpan(0, 0, 3))
                        .ObserveOn(Scheduler.ThreadPool)
                        .ObserveOn(Scheduler.Dispatcher)
                        .Subscribe(time => 
                        {
                            var latitude = ((random.NextDouble() * 0.26114931) - 0.0) + 37.4;
                            var longitude = ((random.NextDouble() * 0.30761719) - 0.0) + 126.8;
                            var pin2 = new GeoCoordinate
                            {
                                Latitude = Convert.ToDouble(latitude),
                                Longitude = Convert.ToDouble(longitude),
                                Speed = 2
                            };
                            geoCollection.Add(pin2);
                        });

                    //위를 막고 여기를 풀어서 100개를 한번에 추가하는 것도 가능
                    //for (int i = 0; i < 100; i++)
                    //{
                    //    var latitude = ((random.NextDouble() * 0.26114931) - 0.0) + 37.4;
                    //    var longitude = ((random.NextDouble() * 0.30761719) - 0.0) + 126.8;
                    //    var pin2 = new GeoCoordinate
                    //    {
                    //        Latitude = Convert.ToDouble(latitude),
                    //        Longitude = Convert.ToDouble(longitude)
                    //    };
                    //    geoCollection.Add(pin2);
                    //}
                    break;
                case "2":
                    //구독취소
                    if(one != null)
                        one.Dispose();

                    //위를 막고 여기를 풀어서 100개를 한번에 지우는 것도 가능
                    //for (int i = 0; i < 100; i++)
                    //{
                    //    geoCollection.Remove(geoCollection.FirstOrDefault());
                    //}
                    break;
            }
        }
    }

    //구글 타일 종류
    public enum GoogleTileTypes
    {
        Hybrid,
        Physical,
        Street,
        Satellite,
        WaterOverlay
    }

    //구글맵 설정 클래스
    public class GoogleTile : TileSource
    {
        private int _server;
        private char _mapmode;
        private GoogleTileTypes _tiletypes;

        public GoogleTileTypes TileTypes
        {
            get { return _tiletypes; }
            set
            {
                _tiletypes = value;
                MapMode = MapModeConverter(value);
            }
        }

        public char MapMode
        {
            get { return _mapmode; }
            set { _mapmode = value; }
        }

        public int Server
        {
            get { return _server; }
            set { _server = value; }
        }

        public GoogleTile()
        {
            UriFormat = @"http://mt{0}.google.com/vt/lyrs={1}&z={2}&x={3}&y={4}";
            Server = 0;
        }

        public override Uri GetUri(int x, int y, int zoomLevel)
        {
            if (zoomLevel > 0)
            {
                var Url = string.Format(UriFormat, Server, MapMode, zoomLevel, x, y);
                return new Uri(Url);
            }
            return null;
        }

        private char MapModeConverter(GoogleTileTypes tiletype)
        {
            switch (tiletype)
            {
                case GoogleTileTypes.Hybrid:
                    {
                        return 'y';
                    }
                case GoogleTileTypes.Physical:
                    {
                        return 't';
                    }
                case GoogleTileTypes.Satellite:
                    {
                        return 's';
                    }
                case GoogleTileTypes.Street:
                    {
                        return 'm';
                    }
                case GoogleTileTypes.WaterOverlay:
                    {
                        return 'r';
                    }
            }
            return ' ';
        }
    }
}